/*
 * q_pds.c		Equivalent Differentiated Service
 *
 * Authors:	Benjamin Gaidioz & Mathieu Goutelle
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <syslog.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <string.h>

#include "utils.h"
#include "tc_util.h"

static void explain(void)
{
  fprintf(stderr,
          "Usage: ... pds limit NUMBER hist_depth NUMBER "
          "nb NUMBER fact_delay NUMBERS fact_loss NUMBERS\n");
}

#define usage() return(-1)

static int pds_parse_opt(struct qdisc_util *qu, int argc, char **argv, struct nlmsghdr *n)
{
  struct tc_pds_qopt opt;
  int nb_set=0, limit_set=0, hist_depth_set = 0, fact_delay_set=0, fact_loss_set=0;
  memset(&opt, 0, sizeof(opt));
	
  while (argc>0) {
    if (!strcmp(*argv,"limit")) {
      NEXT_ARG();
      if(get_size(&opt.limit, *argv)) {
        fprintf(stderr,"Illegal limit:  \"%s\"\n",*argv);
        return -1;
      }
      if (hist_depth_set && opt.limit>opt.history_depth) {
        fprintf(stderr,"\"hist_depth\" must be greater than \"limit\"\n");
        return -1;
      }
      limit_set=1;
    }

    else if (!strcmp(*argv,"nb")) {
      NEXT_ARG();
      if(get_u8(&opt.nb, *argv, 10)) {
        fprintf(stderr,"Illegal number of queues:  \"%s\"\n",*argv);
        return -1;
      }
      nb_set=1;
    }

    else if (!strcmp(*argv,"hist_depth")) {
      NEXT_ARG();
      if(get_size(&opt.history_depth, *argv)) {
        fprintf(stderr,"Illegal history depth:  \"%s\"\n",*argv);
        return -1;
      }
      if (limit_set && opt.limit>opt.history_depth) {
        fprintf(stderr,"\"hist_depth\" must be greater than \"limit\"\n");
        return -1;
      }
      hist_depth_set=1;
    }

    else if (!strcmp(*argv,"fact_delay")) {
      int i;
      if(!nb_set) {
        fprintf(stderr,"You must set \"nb\" before \"fact_delay\"\n");
        return -1;
      }
      opt.factors_delay = malloc(opt.nb*sizeof(__u8));
      for(i=0;i<opt.nb;i++) {
        NEXT_ARG();
        if(get_u8(&opt.factors_delay[i],*argv,10)) {
          fprintf(stderr,"Illegal factor:  \"%s\"\n",*argv);
          return -1;
        }
      }
      fact_delay_set=1;
    }

    else if (!strcmp(*argv,"fact_loss")) {
      int i;
      if (!nb_set) {
        fprintf(stderr,"You must set \"nb\" before \"fact_loss\"\n");
        return -1;
      }
      opt.factors_loss = malloc(opt.nb*sizeof(__u8));
      for (i=0;i<opt.nb;i++) {
        NEXT_ARG();
        if(get_u8(&opt.factors_loss[i],*argv,10)) {
          fprintf(stderr,"Illegal factor:  \"%s\"\n",*argv);
          return -1;
        }
      }
      fact_loss_set=1;
    }

    else {
      fprintf(stderr,"Sorry, I don't understand \"%s\"\n",*argv);
      explain();
      return -1;
    }
    argc--; argv++;
  }
    {
    int k;
    printf("limit = %d, nb = %d, hist_depth = %d,\n",opt.limit, opt.nb, opt.history_depth);
    printf("fact_delay = [");
    for(k=0;k<opt.nb-1;k++) printf("%d, ",opt.factors_delay[k]);
    printf("%d]\n",opt.factors_delay[opt.nb-1]);
    printf("fact_loss = [");
    for(k=0;k<opt.nb-1;k++) printf("%d, ",opt.factors_loss[k]);
    printf("%d]\n",opt.factors_loss[opt.nb-1]);
    }
  if (limit_set && nb_set && hist_depth_set && fact_delay_set && fact_loss_set) {
      printf("%d\n",opt.nb);
    addattr_l(n, 1024, TCA_OPTIONS, &opt, sizeof(opt));
      	  printf("OK\n"); 
    return 0;
  }
  else return -1;
}

static int pds_print_opt(struct qdisc_util *qu, FILE *f, struct rtattr *opt)
{
  struct tc_pds_qopt *qopt;
  int i;

  if (opt == NULL)
    return 0;

  if (RTA_PAYLOAD(opt)  < sizeof(*qopt))
    return -1;

  qopt = RTA_DATA(opt);

  fprintf(f, "limit %up ", qopt->limit);
  fprintf(f, "nb %up ", qopt->nb);
  fprintf(f, "nb %up ", qopt->history_depth);
  fprintf(f, "fact_delay: ");
  for (i=0; i<qopt->nb; i++) fprintf(f, "%u ", qopt->factors_delay[i]);
  fprintf(f, "fact_loss: ");
  for (i=0; i<qopt->nb; i++) fprintf(f, "%u ", qopt->factors_loss[i]);

  return 0;
}

static int pds_print_xstats(struct qdisc_util *qu, FILE *f, struct rtattr *xstats)
{
  return 0;
}


struct qdisc_util pds_util = {
  NULL,
  "pds",
  pds_parse_opt,
  pds_print_opt,
  pds_print_xstats,
};
